#version 120

/*
CrapDeShoes’ shaders, derived from Chocapic13
Place two leading Slashes in front of the following '#define' lines in order to disable an option.
*/

#define SUNLIGHTAMOUNT 0.8				

  #define GODRAYS
  #define GODRAYS_EXPOSURE 0.10
  #define GODRAYS_SAMPLES 6
  #define GODRAYS_DECAY 0.95
  #define GODRAYS_DENSITY 0.65

const float sunPathRotation = -40.0f;

uniform int fogMode;	

uniform sampler2D gcolor;
uniform sampler2D depthtex0;
uniform sampler2D gaux1;

varying vec4 texcoord;
varying vec4 lmcoord;
varying vec3 lightVector;

uniform int worldTime;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;
uniform vec3 sunPosition;

uniform float near;
uniform float far;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;
uniform float aspectRatio;

varying float TimeMidnight;
varying float TimeSunset;
varying float TimeNoon;
varying float TimeSunrise;

varying vec3 sunlight_color;
varying vec3 ambient_color;
varying vec3 skycolor;

varying vec3 sunlight;

float edepth(vec2 coord) { return texture2D(depthtex0,coord).z; }
float luma(vec3 color) { return dot(color.rgb,vec3(0.299, 0.587, 0.114)); }
float ld(float depth) { return (2.0 * near) / (far + near - depth * (far - near)); }

vec2 texel = vec2(1.0/viewWidth,1.0/viewHeight);
vec3 aux = texture2D(gaux1, texcoord.st).rgb;

float	land 			 = aux.b;
vec3	sunPos			 = sunPosition;
vec2 	Texcoord2		 = texcoord.st;
float 	iswater			 = 0.0;
float 	translucent		 = 0.0;
float  pixeldepth = texture2D(depthtex0,texcoord.xy).x;
float totalspec = aux.r*3.0;

float rainx = clamp(rainStrength, 0.0f, 1.0f);
float wetx  = clamp(wetness, 0.0f, 1.0f);

float pw = 1.0/ viewWidth;
float ph = 1.0/ viewHeight;

float sky_lightmap = 1.0;
float torch_lightmap = pow(aux.b,5)*1.5;
float lightning_lightmap = 0.0;
float shadowexit = 0.0;


#ifdef GODRAYS

	float addGodRays(vec2 lightPos,  float nc, vec2 tx, float noise, float noise2, float noise3, float noise4, float noise5) {
			float GDTimeMult = 0.0f;
			if (sunPos.z > 0.0f) {
				sunPos.z = -sunPos.z;
				sunPos.x = -sunPos.x;
				sunPos.y = -sunPos.y;
				GDTimeMult = TimeMidnight;
			} else {
				GDTimeMult = TimeSunrise + TimeNoon + TimeSunset;
			}

		vec2 delta = (tx - lightPos) * GODRAYS_DENSITY / 2.0;
		delta *= -sunPos.z*0.01f;

		float decay = -sunPos.z / 100.0f;
		float colorGD = 0.0f;

			for (int i = 0; i<2;i++) {
		tx -= delta;
		float sample = 0.0f;

      sample = step(texture2D(gaux1, clamp(tx + delta*noise,0.000001,0.999999)).g,0.01);
      sample += step(texture2D(gaux1, clamp(tx + delta*noise2,0.000001,0.999999)).g,0.01);
      sample += step(texture2D(gaux1, clamp(tx + delta*noise3,0.000001,0.999999)).g,0.01);
      sample += step(texture2D(gaux1, clamp(tx + delta*noise4,0.000001,0.999999)).g,0.01);
            sample *= decay;

		colorGD += sample;
		decay *= GODRAYS_DECAY;			
			}
	float bubble = distance(vec2(delta.x*aspectRatio, delta.y), vec2(0.0f, 0.0f))*4.0f;
	bubble = clamp(bubble, 0.0f, 1.0f);
	bubble = 1.0f - bubble;
  
	return (nc + GODRAYS_EXPOSURE * (colorGD*bubble))*GDTimeMult;
	}
	#endif 

void main() {

	vec4 tpos = vec4(sunPos,1.0)*gbufferProjection;
		tpos = vec4(tpos.xyz/tpos.w,1.0);
		vec2 lightPos = tpos.xy/tpos.z;
		lightPos = (lightPos + 1.0f)/2.0f;
			
float rainmask = 0.0;

if(aux.g > 0.1 && aux.g < 0.3) shadowexit = 1.0, land = 1.0;
if(aux.g < 0.01) shadowexit = 1.0;
if(aux.g > 0.01 && aux.g < 0.07) iswater = 1.0, land = 1.0;
if(aux.g > 0.3 && aux.g < 0.5) land = 1.0, translucent = 1.0;
if(aux.g > 0.9) land = 1.0;
if(aux.g > 0.6 && aux.g < 0.75) rainmask = 1.0;


float noiseamp = 0.3f;
					
						
	float width3 = 2.0f;
	float height3 = 2.0f;
	float noiseX3 = ((fract(1.0f-Texcoord2.s*(width3/2.0f))*0.25f)+(fract(Texcoord2.t*(height3/2.0f))*0.75f))*2.0f-1.0f;
					
	noiseX3 = clamp(fract(sin(dot(Texcoord2 ,vec2(18.9898f,28.633f))) * 4378.5453f),0.0f,1.0f)*2.0f-1.0f;
	noiseX3 *= (0.0005f*noiseamp);

	vec4 fragposition = gbufferProjectionInverse * vec4(texcoord.s * 2.0f - 1.0f, texcoord.t * 2.0f - 1.0f, 2.0f * pixeldepth - 1.0f, 1.0f);
	fragposition /= fragposition.w;

	float drawdistance = 0;
	float dist = length(fragposition.xyz);

	float shading = 1.0f;
	float shadingsharp = 1.0f;
	float diffthresh = 0.5;
	
	vec4 worldposition = vec4(0.0);
	vec4 worldpositionraw = vec4(0.0);
			
	worldposition = gbufferModelViewInverse * fragposition;	
	
	float xzDistanceSquared = worldposition.x * worldposition.x + worldposition.z * worldposition.z;
	float yDistanceSquared  = worldposition.y * worldposition.y;
	
	worldpositionraw = worldposition;
	
float ao = 1.0;
			
float sunlight_direct = 1.0;
float sunlight_reflected = 0.0;
float spec = 0.0;
float sky_spec = 0.0;
float fresnel = 0.0;
vec3 specular;
vec3 npos;
float indirect = 0.0;
vec3 Specular_lightmap = vec3(0.0);

vec3 color = texture2D(gcolor, texcoord.st).rgb;

float dampmask = 1.0+ 0.15 * land * wetx;
color = pow(color, vec3(dampmask));
float wspecmask = 0.0;

const float rspread = 0.30f;

float wave = 0.0;

if (land > 0.9 ) {
vec3 torchcolor = vec3(0.6,0.32,0.1);

vec3 Sunlight_lightmap = sunlight_color*SUNLIGHTAMOUNT;
 
vec3 Torchlight_lightmap = torch_lightmap * torchcolor;


vec3 color_sunlight = color * Sunlight_lightmap;
vec3 color_torchlight = color * Torchlight_lightmap;

color_sunlight *= mix(1.0,0.6,TimeMidnight);

color = color_sunlight + color_torchlight;

} else  {
color.rgb = mix(color.rgb,skycolor,rainx)*0.75;
}

float GRa = 0.0f;

#ifdef GODRAYS
	const float grna = 3300.0f;
if (lightPos.x < 1.0 && lightPos.x > 0.0 && lightPos.y < 1.0 && lightPos.y > 0.0) {
			
	GRa = addGodRays(lightPos,0.0f, Texcoord2, noiseX3*grna, noiseX3*grna, noiseX3*grna, noiseX3*grna, noiseX3*grna)/2.0;
}
#endif

color.rgb *= ao;

vec3 rainclr = vec3(0.15,0.15,0.4);
color.rgb = mix(color.rgb,rainclr,rainmask*0.1);

float visiblesun = 0.0;
float temp;
int nb = 0;

if (texcoord.x < pw && texcoord.x < ph && lightPos.x < 1.0 && lightPos.x > 0.0 && lightPos.y < 1.0 && lightPos.y > 0.0) {
	for (int i = 0; i < 16;i++) {
		for (int j = 0; j < 16 ;j++) {
		temp = texture2D(gaux1,lightPos + vec2(pw*(i-8)*7.0,ph*(j-8)*7.0)).g;
		if (temp > 0.04) visiblesun += 0.0;
		else visiblesun += 1.0;
		nb += 1;
		}
	}
	visiblesun /= nb;

}

/* DRAWBUFFERS:NNN3N5 */

    gl_FragData[5] = vec4(wspecmask,wave,GRa,visiblesun);
	gl_FragData[3] = vec4(color, land);
}
